require_dependency 'packages/package_methods'

class External::DropboxSubmissionsController < External::BaseController
  include PackageMethods
  helper :send
  helper :deliveries
  skip_before_filter :check_password_expiry
  prepend_before_filter :check_license
  before_filter :check_send_permissions, :only => [:new, :create]

  def current_user
    ExternalUser.find_by_id(session[:login_user_id])
  end

  def new
     # check if external user has been upgraded
    redirect_to new_dropbox_delivery_path(@dropbox) and return unless current_user.external_email_user?
    super
  end

  def create
    submit # in PackageMethods
  end

  def show
    @delivery = find_workgroup_delivery(params[:id])
    @delivery_send = @delivery.delivery
    @package = @delivery.e_package
    redirect_to page_not_found_path unless @package.e_package_sender == current_user
  end

  def ajax_update_submission_status
    @wg_delivery = find_workgroup_delivery(params[:id])
    render :partial => 'ajax_update_submission_status'
  end

  private

  def check_license
    unless valid_license_or_entitlement?
      redirect_to root_path
    end
  end

  def check_permissions
    @passcode = params[:passcode]
    @membership_or_invitation = EMembership.find_by_dbox_extusr_passcode(@passcode) ||
        Invitation.find_by_passcode(@passcode)

    if @passcode.blank? || @membership_or_invitation.nil?
      redirect_to expired_or_invalid_path and return
    end

    user = @membership_or_invitation.user
    @dropbox = @membership_or_invitation.e_workgroup
    if user.is_a?(ExternalUser)
      session[:login_user_id] = user.id
    end

    # Using a public URL creates an Invitation, manually inviting an outside submitter creates an EMembership
    if !@dropbox.show_public_url? && @membership_or_invitation.is_a?(Invitation)
      redirect_to expired_or_invalid_path and return
    end

    redirect_to expired_or_invalid_path and return if current_user.nil?
    redirect_to expired_or_invalid_path and return if user.home_server != HomeServer.this_server
  end

  def check_package_permissions
    @delivery = Delivery.find_by_id(params[:id])
    if !@delivery || !(current_user.id == @delivery.sender_id)
      redirect_to page_not_found_path
    else
      @external_upload = true
    end
  end

  def check_send_permissions
    error = nil
    if !current_user.uploads_allowed?
      error = "You do not have permission to upload files."
    elsif !current_user.upload_from_ip_allowed?(request.remote_ip)
      error = "You do not have permission to upload files from your IP address."
    elsif @membership_or_invitation.expired?
      redirect_to expired_or_invalid_path and return
    end
    if error
      flash[:error] = error
      redirect_to current_user.external_email_user? ? page_not_found_path : received_index_path
    end
  end

  def transfer_host
    @dropbox.default_transfer_host(request)
  end
end
