class RecipientDelivery < ActiveRecord::Base
  include UrlGeneration
  include ActionController::UrlWriter # for generating paths in notification emails
  include Packages::TransferInfo
  include Packages::FileBrowsing

  belongs_to :e_package
  belongs_to :recipient, :polymorphic => true
  belongs_to :delivery
  belongs_to :share

  has_many :upload_sessions,
    through: :e_package,
    source: :e_sessions,
    include: :transfer_request,
    conditions: [
      "(e_sessions.direction = ? OR e_sessions.direction = ? OR e_sessions.direction = ?) AND e_sessions.share_id = ? AND (transfer_request_id IS NULL OR transfer_requests.flatten = ?)",
      ESession::DIRECTION_UPLOAD,
      ESession::DIRECTION_CREATE,
      ESession::DIRECTION_RELAY,
      '#{share_id}',
      false
    ]
  has_many :package_relay_requests,
    through: :e_package,
    source: :transfer_requests,
    conditions: {direction: ESession::DIRECTION_RELAY, share_id: '#{share_id}', flatten: false}
  has_many :upload_requests,
    through: :e_package,
    source: :transfer_requests,
    conditions: [
      "(direction = ? OR direction = ? OR direction = ?) AND share_id = ? AND flatten = ?",
      ESession::DIRECTION_UPLOAD,
      ESession::DIRECTION_CREATE,
      ESession::DIRECTION_RELAY,
      '#{share_id}',
      false
    ]

  named_scope :public,  conditions: ['private = ?', false]
  named_scope :private, conditions: ['private = ?', true]

  delegate :forward?, :relay?, :sender, :sender_name, :note, :title, :public_recipients,
    :private_recipients, :notifiable_on_upload, :notifiable_on_download, :metadata_xml, :to => :delivery

  attr_accessible :recipient, :recipient_name, :delivery, :e_package, :share, :private

  def default_download_location
    delivery.default_download_location
  end

  def storage_share
    forward? ? delivery.default_destination : share
  end

  def upload_status
    @upload_status ||= begin
      if completed_uploads.any? || forward?
        ESession::STATUS_COMPLETE
      elsif upload_sessions.any?
        # compensate for being stuck in 'willretry' status
        if upload_sessions.last.status == ESession::STATUS_WILLRETRY && upload_timed_out?
          ESession::STATUS_TIMED_OUT
        else
          upload_sessions.last.status
        end
      elsif upload_timed_out?
        ESession::STATUS_TIMED_OUT
      else
        ESession::STATUS_SUBMITTED
      end
    end
  end

end
