# including controller must define downloadable_object, which returns a delivery or recipient_delivery
# to be downloaded
module DownloadableActions

  def self.included base
    base.class_eval do
      before_filter :find_delivery_and_package, only: [:download, :forward, :forward_submit]
      before_filter :ensure_package_downloadable!, only: :download
      include InstanceMethods
    end
  end

  module InstanceMethods

    def download
      create_transfer_url

      case params[:transfer_type]
      when 'http'
        render 'deliveries/http_download'
      when 'connect'
        render 'deliveries/connect_download'
      end
    rescue => e
      flash[:error] = "Download failed, please contact your administrator."
      logger.error("Unable to get transfer spec: #{e}\n#{e.backtrace.join("\n")}")
      render 'deliveries/error'
    end

    def find_delivery_and_package
      @delivery = downloadable_object.delivery
      @package  = downloadable_object.e_package
    rescue => e
      respond_to do |format|
        format.html do
          raise
        end
        format.json do
          render_json_error(:not_found, e.message)
        end
      end
    end

    private

    def create_transfer_url
      # TODO: Handle the case where the storage node has been deleted
      @spec = downloadable_object.get_transfer_spec ESession::DIRECTION_DOWNLOAD,
        params[:items],
        current_user,
        token: true,
        host: transfer_host
      download_spec_hash = JSON.parse(@spec.to_json)
      download_spec_hash["authentication"] = "token"
      download_spec_hash.reject! {|k, v| v.blank?}
      @download_spec_hash = download_spec_hash
      @download_name = generate_download_name params[:items], @package
    end

    def transfer_host
      downloadable_object.default_transfer_host(request) rescue downloadable_object.delivery.default_transfer_host(request)
    end

    def ensure_package_downloadable!
      redirect_to access_denied_url unless @package.can_be_downloaded_by?(current_user, request.remote_ip)
    end

    def generate_download_name items, package
      is_partial_package = partial? items, package
      if is_partial_package && items.size == 1
        name = items.first
      else
        name = package.inner_directory_name
        name += ' (partial)' if is_partial_package
      end
      name
    end

    def partial? items, package
      !package.entire? items
    end
  end
end
