# including class must define storage_share and e_package
module Packages::FileBrowsing

  # /[SHARE PATH]/[package name] - [UUID].aspera-package
  def outer_directory_path(download_path = false)
    if download_path
      default_download_location.node_path(e_package.outer_directory_name)
    else
      storage_share.node_path(e_package.outer_directory_name)
    end
  end

  # PKG - foo
  def inner_directory_name
    EConfiguration.package_path_prefix + e_package.filesafe_title
  end

  # /[SHARE PATH]/[package name] - [UUID].aspera-package/PKG - [package name]
  def inner_directory_path(download_path = false)
    File.join(outer_directory_path(download_path), inner_directory_name)
  end

  def browse_inner_directory
    @browse_inner_directory ||= begin
      result = browse('/')
      if !result.success? && respond_to?(:missing_since) && missing_since.nil?
        self.missing_since = Time.now
        save
      elsif result.success? && respond_to?(:missing_since) && missing_since.present?
        self.missing_since = nil
        save
      end
      result
    end
  end

  def inner_directory_exists?
    if storage_share
      browse_inner_directory.success?
    else
      false
    end
  end

  def node_path(relative_path)
    sanitized_path = Utils.expand_path(relative_path, '/')
    File.join(inner_directory_path, sanitized_path)
  end

  def download_path(relative_path)
    sanitized_path = Utils.expand_path(relative_path, '/')
    File.join(inner_directory_path(true), sanitized_path)
  end

  def package_path(relative_path)
    Utils.relative_path(relative_path, inner_directory_path)
  end

  def browse(relative_path)
    # TODO - a more comprehensive way of paginating browsing so the count parameter is no longer required
    # This attempts to protect against the number of directories exceeding the number of files, which causes some
    # directories to not be displayed. Note that if the number of directories exceeds the number of files by a factor
    # of 2 and is greater than 100, not all directories will be displayed.
    if e_package.e_package_contents_exist?
      count = [e_package.e_package_total_files * 2, 100].max
      storage_share.node.browse(node_path(relative_path), count: count)
    else
      NodeApi::Response::Browse.new(user_message: 'No such file or directory')
    end
  end

  def items_at(relative_path)
    if e_package.e_package_contents_exist?
      browse(relative_path).directory_items
    else
      []
    end
  rescue => e
    logger.warn("attempted to list contents of '#{relative_path}' in #{self.class.name} #{id} but failed")
    logger.debug(e)
    logger.debug(e.backtrace.join("\n"))
    []
  end

end
